from __future__ import unicode_literals

from django_filters import filters
from datetime import timedelta


from django import forms
from django.db.models import Q
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _

LOOKUP_MAP = {
    'icontains': _('contains'),
    'istartswith': _('starts with'),
    'lt': _('less then'),
    'gt': _('grater then'),
    'lte': _('less then or equal'),
    'gte': _('grater then or equal'),
    'exact': '',
    'range': '',
    'in_any': _('in any'),
    'in_all': _('in all'),
    'isnull': _('is NULL'),
    'not_in_any': _('not in any'),
    'not_in_all': _('not in all'),
}


def _truncate(dt):
    return dt.date()


class ModelMultipleChoiceFilter(filters.Filter):
    """
    This filter preforms an OR query on the selected options.
    """
    field_class = forms.MultipleChoiceField

    def filter(self, qs, value):
        value = value or ()
        # check lookup_type in a py2/3 compatible way
        if value and self.lookup_type and not isinstance(self.lookup_type, (str, u"".__class__)):
            value, choosen_lookup_type = value
        else:
            choosen_lookup_type = self.lookup_type

        # it's not true in general if a models field is not required (null=False)
        # if len(value) == len(self.field.choices):
        #     return qs
        q = Q()

        if choosen_lookup_type in ('in_all', 'not_in_all'):
            for v in value:
                # Using Q object does not create all aliases used in inner joins
                qs = qs.filter(**{self.name: v})
        else:
            for v in value:
                q |= Q(**{self.name: v})

        if choosen_lookup_type in ('not_in_any', 'not_in_all'):
            return qs.exclude(q).distinct()
        else:
            return qs.filter(q).distinct()


class DateRangeFilter(filters.DateRangeFilter):

    choices = [
        ('any', _('Any date')),
        ('today', _('Today')),
        ('yesterday', _('Yesterday')),
        ('past7days', _('Past 7 days')),
        ('yesterday', _('Yesterday')),
        ('this_month', _('This month')),
        ('last_month', _('Last month')),
        ('this_year', _('This year')),
        ('last_year', _('Last year')),
    ]

    filters = {  # noqa
        'any': lambda qs, name: qs.all(),
        'today': lambda qs, name: qs.filter(**{
            '%s__year' % name: now().year,
            '%s__month' % name: now().month,
            '%s__day' % name: now().day
        }),
        'yesterday': lambda qs, name: qs.filter(**{
            '%s__year' % name: (now() - timedelta(days=1)).year,
            '%s__month' % name: (now() - timedelta(days=1)).month,
            '%s__day' % name: (now() - timedelta(days=1)).day,
        }),
        'past7days': lambda qs, name: qs.filter(**{
            '%s__gte' % name: _truncate(now() - timedelta(days=7)),
            '%s__lt' % name: _truncate(now() + timedelta(days=1)),
        }),
        'this_month': lambda qs, name: qs.filter(**{
            '%s__year' % name: now().year,
            '%s__month' % name: now().month
        }),
        'last_month': lambda qs, name: qs.filter(**{
            '%s__year' % name: now().year if now().month > 1 else now().year-1,
            '%s__month' % name: now().month-1 if now().month >= 2 else 12,
        }),
        'this_year': lambda qs, name: qs.filter(**{
            '%s__year' % name: now().year,
        }),
        'last_year': lambda qs, name: qs.filter(**{
            '%s__year' % name: now().year-1,
        }),
    }
